﻿using Kivii.Client.Lims.ReportPrinter.Entities;
using Kivii.Client.Lims.ReportPrinter.Request;
using Kivii.Configuration;
using Kivii.GhostScript;
using Kivii.Linq;
using Kivii.Pdf;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Kivii.Client.Lims.ReportPrinter
{
    public partial class FrmMain : Form
    {
        #region 私有变量
        private string _userName = null;
        private Guid _userKvid = Guid.Empty;
        private string _serverUrl = "";
        /// <summary>
        /// Restful通讯客户端
        /// </summary>
        private JsonServiceClient _client = null;
        /// <summary>
        /// 打印设置信息
        /// </summary>
        private PrinterSettings _printerSettings;
        /// <summary>
        /// 打印队列显示
        /// </summary>
        private List<PrintPool> _printJobs = null;
        /// <summary>
        /// 正在打印标记，正在打印时，不允许刷新数据
        /// </summary>
        private bool _isPrinting = false;
        /// <summary>
        /// 打印线程
        /// </summary>
        private Thread _printThread = null;
        /// <summary>
        /// 移除PDF的抬头
        /// </summary>
        //private bool _removePdfHeader = true;

        //private bool _autoCompletePrint = false;
        /// <summary>
        /// 下载报告客户端
        /// </summary>
        private WebClient _downloadClient = null;

        /// <summary>
        /// 设置默认打印机方法
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        [System.Runtime.InteropServices.DllImport("winspool.drv", CharSet = System.Runtime.InteropServices.CharSet.Auto, SetLastError = true)]
        private static extern Boolean SetDefaultPrinter(string name);
        #endregion

        public FrmMain()
        {
            InitializeComponent();
        }

        #region 界面控件默认值管理
        private void setMessage(string msg)
        {
            if (InvokeRequired)
            {
                this.Invoke(new Action(() => { setMessage(msg); }));
                return;
            }
            lblMessage.Text = msg;
            if (msg.StartsWith("成功:")) lblMessage.ForeColor = Color.Green;
            else if (msg.StartsWith("错误:")) lblMessage.ForeColor = Color.Red;
            else if (msg.StartsWith("警告:")) lblMessage.ForeColor = Color.OrangeRed;
            else lblMessage.ForeColor = Color.Gray;
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }
        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Maximized;
            gridPrintJobs.AutoGenerateColumns = false;
            loadDefault(tbxServiceName, tbxUserName, tbxPassword);
            //_client = new JsonServiceClient(_serverUrl);
            _downloadClient = new WebClient();

            PrintDocument print = new PrintDocument();
            _printerSettings = print.PrinterSettings;//默认打印机名
            lblPrinterName.Text = "打印机:" + _printerSettings.PrinterName;
            lblPrinterName.BackColor = Color.Green;
            foreach (string sPrint in PrinterSettings.InstalledPrinters)//获取所有打印机名称
            {
                cbxPrinter.Items.Add(sPrint);
                if (sPrint == _printerSettings.PrinterName) cbxPrinter.SelectedIndex = cbxPrinter.Items.IndexOf(sPrint);
            }
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_client != null) _client.Dispose();
            if (_downloadClient != null) _downloadClient.Dispose();
        }

        private void cbxPrinter_SelectedValueChanged(object sender, EventArgs e)
        {
            var result = SetDefaultPrinter(cbxPrinter.SelectedItem.ToString());
            PrintDocument print = new PrintDocument();
            _printerSettings = print.PrinterSettings;//默认打印机名
            lblPrinterName.Text = "打印机:" + _printerSettings.PrinterName;
            lblPrinterName.BackColor = Color.Green;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (_client != null) return;
            btnConnect.Enabled = false;
            _client = new JsonServiceClient(tbxServiceName.Text);
            _client.Timeout = TimeSpan.FromSeconds(50);//5秒超时
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            //当成功获取到数据后，异步执行以下的代码
            task.Success(onLoginSuccess);
            //当执行错误后，执行以下的代码
            task.Error(onLoginError);
        }

        private void onLoginSuccess(AuthenticateResponse resp)
        {
            setMessage("登录成功：已连接到远程服务器上");
            lblUserInfo.Text = "{0}".Fmt(resp.DisplayName);
            btnRefresh.Enabled = true;
            btnSearch.Enabled = true;
            btnConnect.Enabled = false;
            btnDisconnect.Enabled = true;
            _userName = resp.DisplayName;
            _userKvid = resp.Kvid;
            setDefault(tbxServiceName, tbxUserName, tbxPassword);
        }

        private void onLoginError(Exception ex)
        {
            setMessage($"登陆失败:未能连接到远程服务器,{ex.Message}");
            btnConnect.Enabled = true;
            _client.Dispose();
            _client = null;
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            if (_client == null) return;
            _client.Dispose();
            _client = null;
            btnConnect.Enabled = true;
            tbxPassword.Text = null;
            tbxPassword.Focus();
            lblUserInfo.Text = null;
            setMessage("断开服务：已断开服务器");
        }

        private void FrmMain_Activated(object sender, EventArgs e)
        {
            tbxPassword.Focus();

        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            if (_client == null) return;
            //如果正在打印，不允许刷新数据源
            if (_isPrinting) return;
            btnRefresh.Enabled = false;
            tbSearch.Text = string.Empty;

            var request = new PrintPoolQuery();
            request.IsPrint = false;
            request.OrderBy = "Title";//2020-08-03前台要求按照AW BW排序，即报告号排序
            var task = _client.PostAsync(request);
            task.Success(() =>
            {
                _printJobs = new List<PrintPool>();
                if (cbxJianDu.Checked)
                {
                    _printJobs = task.Result.Results;
                }
                else
                {
                    foreach (var en in task.Result.Results)
                    {
                        if (en.Title.Contains("GJ") || en.Title.Contains("FJ") || en.Title.Contains("TJ")) continue;
                        _printJobs.Add(en);
                    }
                }
                gridPrintJobs.DataSource = _printJobs;

                setMessage($"成功:{DateTime.Now.ToString("HH:mm:ss")}->加载打印列队{_printJobs.Count}条");
                btnRefresh.Enabled = true;

            });
            task.Error((ex) =>
            {
                setMessage($"错误:{DateTime.Now.ToString("HH:mm:ss")}->{ex.Message}");
                btnRefresh.Enabled = true;

            });
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (_client == null) return;
            //如果正在打印，不允许刷新数据源
            if (_isPrinting) return;
            if (tbSearch.Text.IsNullOrEmpty()) return;
            btnSearch.Enabled = false;

            var request = new PrintPoolQuery();
            request.IsPrint = true;
            request.QueryKeys = "Title";
            request.QueryValues = tbSearch.Text;
            var task = _client.PostAsync(request);
            task.Success(() =>
            {
                if (_printJobs.IsNullOrEmpty()) _printJobs = new List<PrintPool>();
                _printJobs.RemoveAll(o => o.IsSearch == false);
                foreach (var item in task.Result.Results)
                {
                    if (_printJobs.Exists(o => o.Kvid == item.Kvid)) continue;
                    if (item.IsPrint)
                    {
                        if (item.Result == "标记为已打印" || item.Count == 0)
                        {
                            item.IsSearch = true;
                            _printJobs.Add(item);
                        }
                    }
                    else _printJobs.Add(item);
                }
                gridPrintJobs.DataSource = null;
                gridPrintJobs.DataSource = _printJobs;

                setMessage($"成功:{DateTime.Now.ToString("HH:mm:ss")}->加载打印列队{_printJobs.Count}条");
                btnSearch.Enabled = true;
                tbSearch.Text = string.Empty;
            });
            task.Error((ex) =>
            {
                setMessage($"错误:{DateTime.Now.ToString("HH:mm:ss")}->{ex.Message}");
                btnSearch.Enabled = true;
                tbSearch.Text = string.Empty;
            });
        }

        private void btnCompletePrintJob_Click(object sender, EventArgs e)
        {
            if (_printJobs.IsNullOrEmpty()) return;
            btnCompletePrintJob.Enabled = false;
            var selectedJobs = _printJobs.Where(o => o.IsSelected).ToList();
            if (selectedJobs == null || selectedJobs.Count == 0)
            {
                selectedJobs = _printJobs.ToList();
            }
            List<PrintPool> removedJobs = new List<PrintPool>();
            foreach (var job in selectedJobs)
            {
                //手动查询的报告说明是已经归档后重新要打的报告  就不再做重复标记的工作了
                if (job.IsSearch)
                {
                    removedJobs.Add(job);
                    continue;
                }
                if (!job.IsPrint) continue;//未打印的不标记
                try
                {
                    var completeRequest = new PrintPoolComplete();
                    completeRequest.Kvid = job.Kvid;
                    completeRequest.Remark = "标记为已打印";
                    completeRequest.OperatorName = _userName;
                    completeRequest.OperatorKvid = _userKvid;
                    _client.Post(completeRequest);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "标记打印任务");
                }

                try
                {
                    var reportPrintAssign = new ReportPrintAssign();
                    reportPrintAssign.StatusType = ReportStatusType.ReportPrint;
                    reportPrintAssign.ReportKvids = new List<Guid>();
                    reportPrintAssign.ReportKvids.Add(job.OwnerKvid);
                    _client.Post(reportPrintAssign);
                }
                catch { }
                removedJobs.Add(job);
            }
            this.Invoke(new Action(() =>
            {
                gridPrintJobs.DataSource = null;
                foreach (var removeJob in removedJobs)
                {
                    _printJobs.Remove(removeJob);
                }
                gridPrintJobs.DataSource = _printJobs;
                btnCompletePrintJob.Enabled = true;
            }));

        }

        private void btnPrint_Click(object sender, EventArgs e)
        {
            if (_printerSettings == null)
            {
                MessageBox.Show("请先设置打印机!");
                return;
            }
            if (_printThread == null)
            {
                btnPrint.Text = "取消打印";
                _printThread = new Thread(printJob);
                _printThread.Start();
            }
            else
            {
                btnPrint.Text = "开始打印";
                _printThread.Abort();
                _printThread = null;
            }
        }

        private void printJob()
        {
            _isPrinting = true;
            setMessage("开始打印...");

            //备注:此变量是为了处理自动标记打印后要移除的项目使用
            //原本写法是标记一个移除一个 但这会导致客户端执行一个一个卡死闪退
            //原因是dataGridView.DataSource绑定了数据源之后不支持移除数据
            //解决办法是 对 dataGridView.DataSource 解绑 即dataGridView.DataSource=null
            //之后再对数组进行增删后绑定到dataGridView.DataSource中 
            var preRemoveJobs = new List<PrintPool>();
            try
            {
                if (_printJobs.IsNullOrEmpty()) return;

                var selectedJobs = _printJobs.Where(o => o.IsSelected).ToList();
                if (selectedJobs.IsNullOrEmpty())
                {
                    selectedJobs = _printJobs;
                }
                foreach (var job in selectedJobs)
                {
                    setMessage($"开始下载PDF文件：{job.Title}");
                    try
                    {
                        bool isPrintSucceed = false;
                        if (job.Count <= 0)
                        {
                            job.IsPrint = true;
                            isPrintSucceed = true;
                            job.PrintStatus = "打印成功";
                            setMessage($"成功:PDF文件无需打印：{job.Title}");
                        }
                        else
                        {
                            var tempDownloadFilePath = KiviiContext.GetTemporaryFilePath(job.Title + ".pdf");
                            string printFilePath = tempDownloadFilePath;//最终要打印的文件路径
                            if (job.FilePath.StartsWith("https")) job.FilePath = job.FilePath.Replace("https", "http");
                            var pdfUrl = job.FilePath;
                            _downloadClient.DownloadFile(pdfUrl, tempDownloadFilePath);
                            if (!File.Exists(tempDownloadFilePath))
                            {
                                job.IsPrint = false;
                                job.PrintStatus = "下载失败";
                                continue;
                            }
                            if (cbxPdfRemoveHeader.Checked )
                            {
                                setMessage($"开始处理PDF文件（移除抬头）：{job.Title}");
                                printFilePath = KiviiContext.GetTemporaryFilePath(job.Title + ".pdf");//实际打印路径变了
                                var removeSetting = new StampSetting();
                                removeSetting.MatchText = "W=800-2000,H=150-400,Y=700-900,R=999";//R=999,移除多次，每页都有抬头，不加只移除首页
                                removeSetting.Pages = "1-E";
                                removeSetting.StampType = StampType.Remove;

                                //2022年11月3日新增 因电子版新增了“电子版”标签，所有电子报告打印时均需移除电子版标签
                                var stampIsSucceed = removeSetting.Stamp(new PdfReader(tempDownloadFilePath), new FileStream(printFilePath, FileMode.Create));
                                if (!stampIsSucceed)//不成功也继续打印
                                {
                                    setMessage($"警告:未能找到PDF文件中的抬头信息：{job.Title}");
                                    job.PrintStatus = "未能移除抬头";
                                    printFilePath = tempDownloadFilePath;
                                }
                            }
                            #region 原写法
                            //List<string> args = new List<string>();
                            ////args.Add("-dQUIET");
                            //args.Add("-dPARANOIDSAFER");
                            //args.Add("-dBATCH");
                            //args.Add("-dNOPAUSE");
                            //args.Add("-r320");
                            ////args.Add($"-dNumCopies={job.Count}");
                            //args.Add("-dNOPROMPT");
                            //args.Add("-sDEVICE=mswinpr2");//使用Windows的打印驱动来
                            ////args.Add("-sDEVICE=ljet4");
                            //args.Add($"-dPDFFitPage");
                            //args.Add($"-dUserArtBox");
                            //args.Add($"-dPrinted");
                            ////args.Add($"-dNumCopies={job.PrintCount}");
                            //args.Add("-dNoCancel");//不显示打印提示窗口
                            //                       //args.Add("-dEmbedAllFonts=true");
                            //                       //args.Add("-r600");
                            //                       //if (!string.IsNullOrEmpty(pages)) args.Add($"-sPageList={pages}");
                            //                       //args.Add(String.Format("-sPAPERSIZE={0}", "a4"));

                            ////args.Add($"-sPDFDocOptions=\"<</NumCopies {job.PrintCount} /Collate true>> setpagedevice\"");
                            ////args.Add($"--saved-pages=\"print copies={job.PrintCount}, normal\"");
                            //args.Add($"-sOutputFile=%printer%{ _printerSettings.PrinterName}");
                            //args.Add(printFilePath);
                            #endregion
                            List<string> args = new List<string>();
                            args.Add("");
                            args.Add("-dQUIET");
                            args.Add("-dNOPAUSE");
                            args.Add("-dPDFFitPage");
                            args.Add("-dBATCH");
                            if (cbxFirstPage.Checked)
                            {
                                args.Add("-dFirstPage=1");
                                args.Add("-dLastPage=1");
                            }
                            args.Add("-c");
                            args.Add($"\"mark /NoCancel true /BitsPerPixel 8 /OutputFile(%printer%{_printerSettings.PrinterName}) /QueryUser 3 /UserSettings<</DocumentName ({job.Title}) /MaxResolution  320>>(mswinpr2) finddevice putdeviceprops setdevice\"");
                            args.Add("-f");
                            args.Add($"\"{printFilePath}\"");


                            //string s = string.Join(" ", args.ToArray());
                            //isPrintSucceed |= Kivii.GhostScript.Tools.Run(args.ToArray()) == 0;

                            for (var i = 0; i < job.Count; i++)
                            {
                                var result = Instance.Run(args.ToArray());
                                isPrintSucceed |= result == 0;
                            }
                        }
                        if (!isPrintSucceed)
                        {
                            job.IsPrint = false;
                            setMessage($"错误:打印PDF文件失败：{job.Title}");
                            job.PrintStatus = "打印失败";
                            continue;
                        }
                        job.IsPrint = true;
                        job.PrintTime = DateTime.Now;
                        job.PrintStatus = "打印成功";
                        setMessage($"成功:PDF文件打印完成：{job.Title}");
                        var markRequest = new PrintPoolMark();
                        markRequest.Kvid = job.Kvid;
                        markRequest.Copies = job.Count;
                        markRequest.Remark = job.PrintStatus;
                        markRequest.OperatorName = _userName;
                        markRequest.OperatorKvid = _userKvid;
                        _client.Post(markRequest);
                        if (cbxPrintAutoComplete.Checked)
                        {
                            //手动查询的报告说明是已经归档后重新要打的报告  就不再做重复标记的工作了
                            if (job.IsSearch)
                            {
                                //_printJobs.Remove(job);
                                preRemoveJobs.Add(job);
                                continue;
                            }
                            try
                            {
                                var completeRequest = new PrintPoolComplete();
                                completeRequest.Kvid = job.Kvid;
                                completeRequest.Remark = "标记为已打印";
                                completeRequest.OperatorName = _userName;
                                completeRequest.OperatorKvid = _userKvid;
                                _client.Post(completeRequest);
                            }
                            catch { }
                            try
                            {
                                var reportPrintAssign = new ReportPrintAssign();
                                reportPrintAssign.StatusType = ReportStatusType.ReportPrint;
                                reportPrintAssign.ReportKvids = new List<Guid>();
                                reportPrintAssign.ReportKvids.Add(job.OwnerKvid);
                                var task = _client.Post(reportPrintAssign);
                            }
                            catch (Exception ex)
                            {
                                WriterLog("/Logs", "/报告下达异常捕获.txt", ex.Message, job.Title);
                            }
                            preRemoveJobs.Add(job);
                            //_printJobs.Remove(job);
                        }
                    }
                    catch (Exception ex)
                    {
                        job.IsPrint = false;
                        job.PrintStatus = "打印异常";
                        try
                        {
                            var markErrorRequest = new PrintPoolMark();
                            markErrorRequest.Kvid = job.Kvid;
                            markErrorRequest.Copies = job.Count;
                            markErrorRequest.Remark = job.PrintStatus;
                            markErrorRequest.OperatorName = _userName;
                            markErrorRequest.OperatorKvid = _userKvid;
                            _client.Post(markErrorRequest);
                        }
                        catch
                        {
                        }
                        setMessage($"错误:打印PDF文件异常：{job.Title}");
                        MessageBox.Show(ex.Message, "打印异常");
                    }
                    finally
                    {
                        this.Invoke(new Action(() =>
                        {
                            gridPrintJobs.DataSource = null;
                            gridPrintJobs.DataSource = _printJobs;

                        }));
                    }
                }

            }
            finally
            {
                setMessage("打印完成...");
                this.Invoke(new Action(() =>
                {
                    btnPrint.Text = "开始打印";
                    _printThread = null;
                    gridPrintJobs.DataSource = null;
                    cbxFirstPage.Checked = false;
                    foreach (var item in preRemoveJobs)
                    {
                        _printJobs.Remove(item);
                    }
                    if (!_printJobs.IsNullOrEmpty()) gridPrintJobs.DataSource = _printJobs;
                }));
                _isPrinting = false;
            }
        }

        private void gridPrintJobs_RowPrePaint(object sender, DataGridViewRowPrePaintEventArgs e)
        {
            gridPrintJobs.Rows[e.RowIndex].DefaultCellStyle.ForeColor = Color.Green;

            var printTime = gridPrintJobs.Rows[e.RowIndex].Cells["PrintTime"].Value;
            var isPrint = gridPrintJobs.Rows[e.RowIndex].Cells["IsPrint"].Value;
            if (isPrint == null) return;
            if (!bool.TryParse(isPrint.ToString(), out var isPrinted)) return;

            //已经标记为已打印显示蓝色
            if (isPrinted) gridPrintJobs.Rows[e.RowIndex].DefaultCellStyle.ForeColor = Color.SteelBlue;
            //已经打印过但没有标记为已打印显示橘色
            else if (printTime != null)
            {
                gridPrintJobs.Rows[e.RowIndex].DefaultCellStyle.ForeColor = Color.Tomato;
            }
        }

        private void tbSearch_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode != Keys.Enter) return;
            btnSearch_Click(null, null);
        }

        private void tbxPassword_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode != Keys.Enter) return;
            if (_client != null) return;
            btnConnect_Click(null, null);
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            if (_printJobs.IsNullOrEmpty()) return;
            foreach (var item in _printJobs)
            {
                item.IsSelected = true;
            }
            gridPrintJobs.DataSource = null;
            gridPrintJobs.DataSource = _printJobs;
        }

        private void btnSelectCancel_Click(object sender, EventArgs e)
        {
            if (_printJobs.IsNullOrEmpty()) return;
            foreach (var item in _printJobs)
            {
                item.IsSelected = false;
            }
            gridPrintJobs.DataSource = null;
            gridPrintJobs.DataSource = _printJobs;
        }

        private void btnChangeQuantity_Click(object sender, EventArgs e)
        {
            if (_printJobs.IsNullOrEmpty()) return;
            var quantity = (int)nudQuantity.Value;
            var selectedJobs = _printJobs.Where(o => o.IsSelected).ToList();
            if (selectedJobs.Count <= 0)
            {
                MessageBox.Show("请先批量选中需要修改份数的报告!");
                return;
            }
            if (MessageBox.Show($"当前选中报告数: {selectedJobs.Count}份\n当前设定份数: {nudQuantity.Value}份\n即将调整份数: {quantity}份", "确认以继续", MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification) == DialogResult.Yes)
            {
                foreach (var item in selectedJobs)
                {
                    item.Count = quantity;
                }
                gridPrintJobs.DataSource = null;
                gridPrintJobs.DataSource = _printJobs;
            }
        }

        //往本地写文件
        public void WriterLog(string path, string logName, string log, string page)
        {
            System.DateTime tm = DateTime.Now;
            string pathStr = AppDomain.CurrentDomain.BaseDirectory + path;
            string p = pathStr + logName;
            if (!Directory.Exists(pathStr))//判断路径下目录是否存在
            {
                Directory.CreateDirectory(pathStr);
                if (!File.Exists(p))
                {
                    using (StreamWriter sw = File.CreateText(p))
                    {
                        sw.WriteLine(log + "----------" + tm + "[" + page + "]");
                    }

                }
                else
                {
                    using (StreamWriter sw = File.AppendText(p))
                    {
                        sw.WriteLine(log + "----------" + tm + "[" + page + "]");
                    }
                }
            }
            else
            {
                if (!File.Exists(p))
                {
                    using (StreamWriter sw = File.CreateText(p))
                    {
                        sw.WriteLine(log + "----------" + tm + "[" + page + "]");
                    }

                }
                else
                {
                    using (StreamWriter sw = File.AppendText(p))
                    {
                        sw.WriteLine(log + "----------" + tm + "[" + page + "]");
                    }
                }

            }
        }
    }
}
