﻿using Kivii.Client.Lims.ReportPrinter.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Kivii.Client.Lims.ReportPrinter.Request
{
    public class PrintPoolQueryResponse : RestfulQueryResponse<PrintPool> { }

    [Route(Configs.RoutePrintPoolQuery)]
    public class PrintPoolQuery : IReturn<PrintPoolQueryResponse>
    {
        public string EntityType { get; set; } = "Kivii.Lims.Entities.Report";
        public bool IsPrint { get; set; }
        public string Category { get; set; }
        public Guid OperatorKvid { get; set; }
        public string QueryKeys { get; set; }
        public string QueryValues { get; set; }
        public string OrderBy { get; set; }
    }

    public class PrintPoolCreateResponse : RestfulCreateResponse<PrintPool> { }

    [Route(Configs.RoutePrintPoolCreate)]
    public class PrintPoolCreate : IReturn<PrintPoolCreateResponse>
    {
        public string EntityType { get; set; } = "Kivii.Lims.Entities.Report";
        public PrintPool Item { get; set; }
    }

    public class PrintPoolMarkResponse : RestfulQueryResponse<PrintPool> { }

    [Route(Configs.RoutePrintPoolMark)]
    public class PrintPoolMark : IReturn<PrintPoolMarkResponse>
    {
        public string EntityType { get; set; } = "Kivii.Lims.Entities.Report";

        public Guid Kvid { get; set; }
        public int Copies { get; set; }
        public string Remark { get; set; }
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }
    }

    public class PrintPoolCompleteResponse : RestfulQueryResponse<PrintPool> { }

    [Route(Configs.RoutePrintPoolComplete)]
    public class PrintPoolComplete : IReturn<PrintPoolCompleteResponse>
    {
        public string EntityType { get; set; } = "Kivii.Lims.Entities.Report";

        public Guid Kvid { get; set; }
        public string Remark { get; set; }
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }
    }

    public enum ReportStatusType
    {
        Unsupported = 0,

        /// <summary>
        /// 委托受理
        /// </summary>
        CommissionAccept = 100,

        /// <summary>
        /// 合同评审
        /// </summary>
        ContractReview = 200,

        /// <summary>
        /// 任务分派
        /// </summary>
        TaskAssign = 300,

        /// <summary>
        /// 数据录入
        /// </summary>
        DataEntry = 400,

        /// <summary>
        /// 报告编制
        /// </summary>
        ReportPreparation = 500,

        /// <summary>
        /// 报告审核
        /// </summary>
        ReportReview = 600,

        /// <summary>
        /// 报告签发
        /// </summary>
        ReportIssue = 700,

        /// <summary>
        /// 报告打印
        /// </summary>
        ReportPrint = 800,

        /// <summary>
        /// 报告归档
        /// </summary>
        ReportCollected = int.MaxValue
    }


    public class ReportPrintAssignResponse : RestfulQueryResponse<Guid> { }

    [Route(Configs.RouteReportPrintAssign)]
    public class ReportPrintAssign : IReturn<ReportPrintAssignResponse>
    {
        public List<Guid> ReportKvids { get; set; }

        public ReportStatusType StatusType { get; set; }
    }
}
